@@ -1,18 +1,17 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2015-2019 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2015-2020 -- leonerd@leonerd.org.uk
 
-package Device::Chip::MPL3115A2;
+use v5.26;
+use Object::Pad 0.19;
 
-use strict;
-use warnings;
-use base qw( Device::Chip::Base::RegisteredI2C );
+package Device::Chip::MPL3115A2 0.07;
+class Device::Chip::MPL3115A2
+   extends Device::Chip::Base::RegisteredI2C;
 
 use utf8;
 
-our $VERSION = '0.06';
-
 use Carp;
 
 use Future::AsyncAwait;
@@ -28,12 +27,13 @@ C<Device::Chip::MPL3115A2> - chip driver for a F<MPL3115A2>
 =head1 SYNOPSIS
 
    use Device::Chip::MPL3115A2;
+   use Future::AsyncAwait;
 
    my $chip = Device::Chip::MPL3115A2->new;
-   $chip->mount( Device::Chip::Adapter::...->new )->get;
+   await $chip->mount( Device::Chip::Adapter::...->new );
 
    printf "Current pressure is %.2f kPa\n",
-      $chip->read_pressure->get;
+      await $chip->read_pressure;
 
 =head1 DESCRIPTION
 
@@ -47,7 +47,7 @@ concepts or features, only the use of this module to access them.
 
 =cut
 
-sub I2C_options
+method I2C_options
 {
    return (
       # This device has a constant address
@@ -129,36 +129,36 @@ bitfield { format => "bytes-LE" }, CTRL_REG =>
    PP_OD2 => boolfield( 21 );
 
 # Converted pressure
-async sub _mplread_p
+async method _mplread_p ( $reg )
 {
-   my $v = unpack "L>", "\0" . await $_[0]->read_reg( $_[1], 3 );
+   my $v = unpack "L>", "\0" . await $self->read_reg( $reg, 3 );
    return $v / 64
 }
 
 # Converted altitude
-async sub _mplread_a
+async method _mplread_a ( $reg )
 {
-   my ( $msb, $lsb ) = unpack "s>C", await $_[0]->read_reg( $_[1], 3 );
+   my ( $msb, $lsb ) = unpack "s>C", await $self->read_reg( $reg, 3 );
    return $msb + ( $lsb / 256 );
 }
 
 # Converted temperature
-async sub _mplread_t
+async method _mplread_t ( $reg )
 {
-   my ( $msb, $lsb ) = unpack "cC", await $_[0]->read_reg( $_[1], 2 );
+   my ( $msb, $lsb ) = unpack "cC", await $self->read_reg( $reg, 2 );
    return $msb + ( $lsb / 256 );
 }
 
 =head1 ACCESSORS
 
-The following methods documented with a trailing call to C<< ->get >> return
-L<Future> instances.
+The following methods documented in an C<await> expression return L<Future>
+instances.
 
 =cut
 
 =head2 read_config
 
-   $config = $chip->read_config->get
+   $config = await $chip->read_config;
 
 Returns a C<HASH> reference of the contents of control registers C<CTRL_REG1>
 to C<CTRL_REG3>, using fields named from the data sheet.
@@ -181,7 +181,7 @@ to C<CTRL_REG3>, using fields named from the data sheet.
 
 =head2 change_config
 
-   $chip->change_config( %changes )->get
+   await $chip->change_config( %changes );
 
 Writes updates to the control registers C<CTRL_REG1> to C<CTRL_REG3>. This
 will be performed as a read-modify-write operation, so any fields not given
@@ -193,30 +193,25 @@ subsequent modifications more efficient. This cache will not respect the
 
 =cut
 
-async sub _cached_read_ctrlreg
-{
-   my $self = shift;
+has $_configbytes;
 
-   return $self->{configbytes} //= await $self->read_reg( REG_CTRL_REG1, 3 );
+async method _cached_read_ctrlreg ()
+{
+   return $_configbytes //= await $self->read_reg( REG_CTRL_REG1, 3 );
 }
 
-async sub read_config
+async method read_config ()
 {
-   my $self = shift;
-
    return { unpack_CTRL_REG( await $self->_cached_read_ctrlreg ) };
 }
 
-async sub change_config
+async method change_config ( %changes )
 {
-   my $self = shift;
-   my %changes = @_;
-
    my $config = await $self->read_config;
 
    $config->{$_} = $changes{$_} for keys %changes;
 
-   my $bytes = $self->{configbytes} = pack_CTRL_REG( %$config );
+   my $bytes = $_configbytes = pack_CTRL_REG( %$config );
 
    await $self->write_reg( REG_CTRL_REG1, $bytes );
 }
@@ -225,9 +220,9 @@ async sub change_config
 
 =head2 set_sealevel_pressure
 
-   $pressure = $chip->get_sealevel_pressure->get
+   $pressure = await $chip->get_sealevel_pressure;
 
-   $chip->set_sealevel_pressure->get( $pressure )
+   await $chip->set_sealevel_pressure( $pressure );
 
 Read or write the barometric pressure calibration register which is used to
 convert pressure to altitude when the chip is in altimeter mode, in Pascals.
@@ -235,17 +230,19 @@ The default value is 101,326 Pa.
 
 =cut
 
-async sub get_sealevel_pressure {
-   unpack( "S<", await $_[0]->read_reg( REG_BAR_IN_MSB, 2 ) ) * 2;
+async method get_sealevel_pressure ()
+{
+   unpack( "S<", await $self->read_reg( REG_BAR_IN_MSB, 2 ) ) * 2;
 }
 
-sub set_sealevel_pressure {
-   $_[0]->write_reg( REG_BAR_IN_MSB, pack "S<", $_[1] / 2 )
+async method set_sealevel_pressure ( $pressure )
+{
+   await $self->write_reg( REG_BAR_IN_MSB, pack "S<", $pressure / 2 )
 }
 
 =head2 read_pressure
 
-   $pressure = $chip->read_pressure->get
+   $pressure = await $chip->read_pressure;
 
 Returns the value of the C<OUT_P_*> registers, suitably converted into
 Pascals. (The chip must be in barometer mode and must I<not> be in C<RAW> mode
@@ -253,11 +250,11 @@ for the conversion to work).
 
 =cut
 
-sub read_pressure { shift->_mplread_p( REG_OUT_P_MSB ) }
+async method read_pressure () { return await $self->_mplread_p( REG_OUT_P_MSB ) }
 
 =head2 read_altitude
 
-   $altitude = $chip->read_altitude->get
+   $altitude = await $chip->read_altitude;
 
 Returns the value of the C<OUT_P_*> registers, suitably converted into metres.
 (The chip must be in altimeter mode and must I<not> be in C<RAW> mode for the
@@ -265,26 +262,26 @@ conversion to work).
 
 =cut
 
-sub read_altitude { shift->_mplread_a( REG_OUT_P_MSB ) }
+async method read_altitude () { return await $self->_mplread_a( REG_OUT_P_MSB ) }
 
 =head2 read_temperature
 
-   $temperature = $chip->read_temperature->get
+   $temperature = await $chip->read_temperature;
 
 Returns the value of the C<OUT_T_*> registers, suitable converted into degrees
 C. (The chip must I<not> be in C<RAW> mode for the conversion to work).
 
 =cut
 
-sub read_temperature { shift->_mplread_t( REG_OUT_T_MSB ) }
+async method read_temperature () { return await $self->_mplread_t( REG_OUT_T_MSB ) }
 
 =head2 read_min_pressure
 
 =head2 read_max_pressure
 
-   $pressure = $chip->read_min_pressure->get
+   $pressure = await $chip->read_min_pressure;
 
-   $pressure = $chip->read_max_pressure->get
+   $pressure = await $chip->read_max_pressure;
 
 Returns the values of the C<P_MIN> and C<P_MAX> registers, suitably converted
 into Pascals.
@@ -293,28 +290,28 @@ into Pascals.
 
 =head2 clear_max_pressure
 
-   $chip->clear_min_pressure->get
+   await $chip->clear_min_pressure;
 
-   $chip->clear_max_pressure->get
+   await $chip->clear_max_pressure;
 
 Clear the C<P_MIN> or C<P_MAX> registers, resetting them to start again from
 the next measurement.
 
 =cut
 
-sub read_min_pressure { shift->_mplread_p( REG_P_MIN_MSB ) }
-sub read_max_pressure { shift->_mplread_p( REG_P_MAX_MSB ) }
+async method read_min_pressure () { return await $self->_mplread_p( REG_P_MIN_MSB ) }
+async method read_max_pressure () { return await $self->_mplread_p( REG_P_MAX_MSB ) }
 
-sub clear_min_pressure { shift->write_reg( REG_P_MIN_MSB, "\x00\x00\x00" ) }
-sub clear_max_pressure { shift->write_reg( REG_P_MAX_MSB, "\x00\x00\x00" ) }
+async method clear_min_pressure () { return await $self->write_reg( REG_P_MIN_MSB, "\x00\x00\x00" ) }
+async method clear_max_pressure () { return await $self->write_reg( REG_P_MAX_MSB, "\x00\x00\x00" ) }
 
 =head2 read_min_altitude
 
 =head2 read_max_altitude
 
-   $altitude = $chip->read_min_altitude->get
+   $altitude = await $chip->read_min_altitude;
 
-   $altitude = $chip->read_max_altitude->get
+   $altitude = await $chip->read_max_altitude;
 
 Returns the values of the C<P_MIN> and C<P_MAX> registers, suitably converted
 into metres.
@@ -325,17 +322,17 @@ into metres.
 
 =head2 clear_max_altitude
 
-   $chip->clear_min_altitude->get
+   await $chip->clear_min_altitude;
 
-   $chip->clear_max_altitude->get
+   await $chip->clear_max_altitude;
 
 Clear the C<P_MIN> or C<P_MAX> registers, resetting them to start again from
 the next measurement.
 
 =cut
 
-sub read_min_altitude { shift->_mplread_a( REG_P_MIN_MSB ) }
-sub read_max_altitude { shift->_mplread_a( REG_P_MAX_MSB ) }
+async method read_min_altitude () { return await $self->_mplread_a( REG_P_MIN_MSB ) }
+async method read_max_altitude () { return await $self->_mplread_a( REG_P_MAX_MSB ) }
 
 *clear_min_altitude = \&clear_min_pressure;
 *clear_max_altitude = \&clear_max_pressure;
@@ -344,9 +341,9 @@ sub read_max_altitude { shift->_mplread_a( REG_P_MAX_MSB ) }
 
 =head2 read_max_temperature
 
-   $temperature = $chip->read_min_temperature->get
+   $temperature = await $chip->read_min_temperature;
 
-   $temperature = $chip->read_max_temperature->get
+   $temperature = await $chip->read_max_temperature;
 
 Returns the values of the C<T_MIN> and C<T_MAX> registers, suitably converted
 into metres.
@@ -355,20 +352,20 @@ into metres.
 
 =head2 clear_max_temperature
 
-   $chip->clear_min_temperature->get
+   await $chip->clear_min_temperature;
 
-   $chip->clear_max_temperature->get
+   await $chip->clear_max_temperature;
 
 Clear the C<T_MIN> or C<T_MAX> registers, resetting them to start again from
 the next measurement.
 
 =cut
 
-sub read_min_temperature { shift->_mplread_t( REG_T_MIN_MSB ) }
-sub read_max_temperature { shift->_mplread_t( REG_T_MAX_MSB ) }
+async method read_min_temperature () { return await $self->_mplread_t( REG_T_MIN_MSB ) }
+async method read_max_temperature () { return await $self->_mplread_t( REG_T_MAX_MSB ) }
 
-sub clear_min_temperature { shift->write_reg( REG_T_MIN_MSB, "\x00\x00" ) }
-sub clear_max_temperature { shift->write_reg( REG_T_MAX_MSB, "\x00\x00" ) }
+async method clear_min_temperature () { return await $self->write_reg( REG_T_MIN_MSB, "\x00\x00" ) }
+async method clear_max_temperature () { return await $self->write_reg( REG_T_MAX_MSB, "\x00\x00" ) }
 
 =head1 METHODS
 
@@ -376,17 +373,15 @@ sub clear_max_temperature { shift->write_reg( REG_T_MAX_MSB, "\x00\x00" ) }
 
 =head2 check_id
 
-   $chip->check_id->get
+   await $chip->check_id;
 
 Reads the C<WHO_AM_I> register and checks for a valid ID result. The returned
 future fails if the expected result is not received.
 
 =cut
 
-async sub check_id
+async method check_id ()
 {
-   my $self = shift;
-
    my $val = await $self->read_reg( REG_WHO_AM_I, 1 );
 
    my $id = unpack "C", $val;
@@ -399,7 +394,7 @@ async sub check_id
 
 =head2 start_oneshot
 
-   $chip->start_oneshot->get
+   await $chip->start_oneshot;
 
 Sets the C<OST> bit of C<CTRL_REG1> to start a one-shot measurement when in
 standby mode. After calling this method you will need to use
@@ -408,10 +403,8 @@ the interrupts.
 
 =cut
 
-async sub start_oneshot
+async method start_oneshot ()
 {
-   my $self = shift;
-
    my $bytes = await $self->_cached_read_ctrlreg;
 
    my $ctrl_reg1 = substr( $bytes, 0, 1 ) | "\x02"; # Set OST bit
@@ -420,16 +413,14 @@ async sub start_oneshot
 
 =head2 busywait_oneshot
 
-   $chip->busywait_oneshot->get
+   await $chip->busywait_oneshot;
 
 Repeatedly reads the C<OST> bit of C<CTRL_REG1> until it becomes clear.
 
 =cut
 
-async sub busywait_oneshot
+async method busywait_oneshot ()
 {
-   my $self = shift;
-
    while(1) {
       my $ctrl_reg1 = await $self->read_reg( REG_CTRL_REG1, 1 );
       last if not( ord( $ctrl_reg1 ) & 0x02 );
@@ -438,16 +429,14 @@ async sub busywait_oneshot
 
 =head2 oneshot
 
-   $chip->oneshot->get
+   await $chip->oneshot;
 
 A convenient wrapper around C<start_oneshot> and C<busywait_oneshot>.
 
 =cut
 
-async sub oneshot
+async method oneshot ()
 {
-   my $self = shift;
-
    await $self->start_oneshot;
    await $self->busywait_oneshot;
 }
